searchData={"content_type":"text/markdown","items":[{"doc":"Posexional is a library to manage positional files in Elixir.\n\n\n\nPositional files are the most bad and **terrifying file format** you ever want\nto work with, believe me, it's utter crap. Still some *web services* use this\nformat to expose data services, and if you are here reading this, probably you\nare in luck like us!\n\nA positional file has a *format specification* like this:\n\n| field name | offset | length | notes                                           |\n|------------|--------|--------|-------------------------------------------------|\n| code       | 0      | 5      | request code fill with 0 and align on the right |\n| prog       | 0      | 5      | progressive number (fill with spaces)           |\n| type       | 21     | 2      | AA                                              |\n| number     | 24     | 2      | 01                                              |\n| name       | 27     | 10     | person name (fill with -)                       |\n| future use | 38     | 2      | leave empty                                     |\n| end        | 41     | 1      | !                                               |\n\nThe expected results should be something like this:\n\n    000B1    1AA01george----  !\n    000B2    2AA01john------  !\n    000B3    3AA01ringo-----  !\n    000B4    4AA01paul------  !\n\n\nCool uh?\n\nWith Posexional you can produce this file by defining a module that use\nPosexional.\n\n```elixir\ndefmodule BeatlesFile do\n  use PosexionalFile\n\n  @separator \"\\\\n\"\n\n  row :beatles do\n    value :code, 5, filler: ?0, alignment: :right\n    progressive_number :code, 5\n    fixed_value \"AA\"\n    fixed_value \"01\"\n    value :name, 10, filler: ?-\n    empty 2\n    fixed_value \"!\"\n  end\nend\n```\n\nAnd then use it in your code.\n\n```elixir\nBeatlesFile.write([\n  beatles: [code: \"B1\", name: \"george\"],\n  beatles: [code: \"B2\", name: \"john\"],\n  beatles: [code: \"B2\", name: \"ringo\"],\n  beatles: [code: \"B2\", name: \"paul\"]\n])\n```\n\nIn the first part we **define the structure** inside a module. We are not\nsaying what the content or the number of rows there will be, we are just saying\nthat there is a row called `:beatles` with the structure declared by the fields.\n\nThen we can call `BeatlesFile.write/1`, we pass the data that should be written\nin the fields. **Just the relevant data**, The empty fields, the fixed values\nor the progressive number is managed by the library itself.\n\nThe `write/1` function accept a keyword list with the row name as key, and a\nkeyword list of {field name, field value} of data. If some data is bigger than\nthe field size an error is thrown\n\nWith the same exact module, we can even **read a positional file** by calling\n`read/1` and passing a binary string of the file content.\n\nThere is only one thing to notice, when we write a file we **can be\ndeclarative** and say what row we want to write, as well as the data we want in\nit. On the other hand, while reading a positional file, we don't know which\nrows we are reading, so we need to tell in some way to every row how it is\nrecognized, so that the parser is able to do its job.\n\nSince we only have a row type (the beatles one) we can just say to the module\nto always match the beatles row, as simple as:\n\n```elixir\ndefmodule BeatlesFile do\n  use PosexionalFile\n\n  @separator \"\\\\n\"\n\n  row :beatles, :always do # add :always here to always match this row while reading\n    value :code, 5, filler: ?0, alignment: :right\n    progressive_number :code, 5\n    fixed_value \"AA\"\n    fixed_value \"01\"\n    value :name, 10, filler: ?-\n    empty 2\n    fixed_value \"!\"\n  end\nend\n```\n\n`:always` is a special type that always match, you could also pass `:never`\n(not so useful!) and, to gain total control over the choice, a function with a\nsingle argument(the full row content) to match some data inside of it.\n\nNow we are able to parse a positional file.\n\n```elixir\n\"000B1    1AA01george----  !\\n000B2    2AA01john------  !\\n000B2    3AA01ringo-----  !\\n000B2    4AA01paul------  !\"\n|> BeatlesFile.read\n|> IO.puts\n```\n\nThis is the output.\n\n```elixir\n[beatles: [code: \"B1\", code: 1, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"george\", fixed_value: \"!\"],\n beatles: [code: \"B2\", code: 2, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"john\", fixed_value: \"!\"],\n beatles: [code: \"B2\", code: 3, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"ringo\", fixed_value: \"!\"],\n beatles: [code: \"B2\", code: 4, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"paul\", fixed_value: \"!\"]]\n```\n\nPositional files cool again!!! Well no...they still sucks...but a little less.","ref":"Posexional.html","title":"Posexional","type":"module"},{"doc":"read a positional stream of data with the given struct, returns a keyword list of the extracted data","ref":"Posexional.html#read/2","title":"Posexional.read/2","type":"function"},{"doc":"same as read/2, but with a path to a file to read the stream from","ref":"Posexional.html#read_file!/2","title":"Posexional.read_file!/2","type":"function"},{"doc":"","ref":"Posexional.html#stream!/2","title":"Posexional.stream!/2","type":"function"},{"doc":"write a positional file with the given struct and data","ref":"Posexional.html#write/2","title":"Posexional.write/2","type":"function"},{"doc":"same as write/2, but with a path to a new file to write the result to","ref":"Posexional.html#write_file!/3","title":"Posexional.write_file!/3","type":"function"},{"doc":"generic utility functions for fields","ref":"Posexional.Field.html","title":"Posexional.Field","type":"module"},{"doc":"true if the value passed contains only the filler value","ref":"Posexional.Field.html#contains_only?/2","title":"Posexional.Field.contains_only?/2","type":"function"},{"doc":"","ref":"Posexional.Field.html#depositionalize/2","title":"Posexional.Field.depositionalize/2","type":"function"},{"doc":"nil if the value is an empty string, or a string containing only the filler","ref":"Posexional.Field.html#nil_if_empty/2","title":"Posexional.Field.nil_if_empty/2","type":"function"},{"doc":"justify a value given alignment, size and filler char, if the given value\nis longer it gets trimmed","ref":"Posexional.Field.html#positionalize/2","title":"Posexional.Field.positionalize/2","type":"function"},{"doc":"iex> Posexional.Field.positionalize(\"test\",\n    ...>   %{alignment: :left, size: 10, filler: ?\\s})\n    \"test      \"\n\n    iex> Posexional.Field.positionalize(\"test\",\n    ...>   %{alignment: :right, size: 10, filler: ?\\s})\n    \"      test\"\n\n    iex> Posexional.Field.positionalize(\"test\",\n    ...>   %{alignment: :right, size: 5, filler: ?\\s})\n    \" test\"\n\n    iex> Posexional.Field.positionalize(\"test\",\n    ...>   %{alignment: :right, size: 5, filler: ?-})\n    \"-test\"\n\n    iex> Posexional.Field.positionalize(\"testtest\",\n    ...>   %{alignment: :right, size: 5, filler: ?-})\n    \"testt\"","ref":"Posexional.Field.html#positionalize/2-examples","title":"Examples - Posexional.Field.positionalize/2","type":"function"},{"doc":"this module represent a single field in a row of a positional file without a value","ref":"Posexional.Field.Empty.html","title":"Posexional.Field.Empty","type":"module"},{"doc":"","ref":"Posexional.Field.Empty.html#new/2","title":"Posexional.Field.Empty.new/2","type":"function"},{"doc":"outputs an empty field","ref":"Posexional.Field.Empty.html#write/1","title":"Posexional.Field.Empty.write/1","type":"function"},{"doc":"iex> Posexional.Field.Empty.write(%Posexional.Field.Empty{filler: ?-, size: 10})\n    \"----------\"\n\n    iex> Posexional.Field.Empty.write(%Posexional.Field.Empty{filler: ?\\s, size: 2})\n    \"  \"","ref":"Posexional.Field.Empty.html#write/1-examples","title":"Examples - Posexional.Field.Empty.write/1","type":"function"},{"doc":"","ref":"Posexional.Field.Empty.html#t:t/0","title":"Posexional.Field.Empty.t/0","type":"type"},{"doc":"this module represent a fixed value field\nit is initialized with its value and it never change","ref":"Posexional.Field.FixedValue.html","title":"Posexional.Field.FixedValue","type":"module"},{"doc":"","ref":"Posexional.Field.FixedValue.html#new/1","title":"Posexional.Field.FixedValue.new/1","type":"function"},{"doc":"this module represent a single field in a row of a positional file with a progressive number","ref":"Posexional.Field.ProgressiveNumber.html","title":"Posexional.Field.ProgressiveNumber","type":"module"},{"doc":"","ref":"Posexional.Field.ProgressiveNumber.html#new/3","title":"Posexional.Field.ProgressiveNumber.new/3","type":"function"},{"doc":"","ref":"Posexional.Field.ProgressiveNumber.html#write/2","title":"Posexional.Field.ProgressiveNumber.write/2","type":"function"},{"doc":"","ref":"Posexional.Field.ProgressiveNumber.html#t:t/0","title":"Posexional.Field.ProgressiveNumber.t/0","type":"type"},{"doc":"this module represent a single field in a row of a positional file","ref":"Posexional.Field.TypedField.html","title":"Posexional.Field.TypedField","type":"module"},{"doc":"","ref":"Posexional.Field.TypedField.html#new/4","title":"Posexional.Field.TypedField.new/4","type":"function"},{"doc":"","ref":"Posexional.Field.TypedField.html#read/2","title":"Posexional.Field.TypedField.read/2","type":"function"},{"doc":"","ref":"Posexional.Field.TypedField.html#t:field_type/0","title":"Posexional.Field.TypedField.field_type/0","type":"type"},{"doc":"","ref":"Posexional.Field.TypedField.html#t:t/0","title":"Posexional.Field.TypedField.t/0","type":"type"},{"doc":"this module represent a single field in a row of a positional file","ref":"Posexional.Field.Value.html","title":"Posexional.Field.Value","type":"module"},{"doc":"","ref":"Posexional.Field.Value.html#new/3","title":"Posexional.Field.Value.new/3","type":"function"},{"doc":"outputs a field","ref":"Posexional.Field.Value.html#write/2","title":"Posexional.Field.Value.write/2","type":"function"},{"doc":"iex> Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 5), \"test\")\n    \"test \"\n\n    iex> Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 5), \"too long\")\n    ** (RuntimeError) The value too long is too long for the test field. The maximum size is 5 while the value is 8\n\n    iex> Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10), \"test\")\n    \"test      \"\n\n    iex> Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10), \"test\")\n    \"test      \"\n\n    iex> Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10, filler: ?0), \"test\")\n    \"test000000\"\n\n    iex> Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10, filler: ?0, alignment: :right), \"test\")\n    \"000000test\"\n\n    iex> Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10), 50)\n    ** (RuntimeError) The value provided for the test field doesn't seem to be a string","ref":"Posexional.Field.Value.html#write/2-examples","title":"Examples - Posexional.Field.Value.write/2","type":"function"},{"doc":"","ref":"Posexional.Field.Value.html#t:t/0","title":"Posexional.Field.Value.t/0","type":"type"},{"doc":"a Posexional.File is the main struct to manage a positional file","ref":"Posexional.File.html","title":"Posexional.File","type":"module"},{"doc":"","ref":"Posexional.File.html#find_row/2","title":"Posexional.File.find_row/2","type":"function"},{"doc":"","ref":"Posexional.File.html#get_counters/1","title":"Posexional.File.get_counters/1","type":"function"},{"doc":"adds a generator for every progressive_number_field in the file.\n\nThe fields are grouped by name, so that you can specify many counters for every row","ref":"Posexional.File.html#manage_counters/1","title":"Posexional.File.manage_counters/1","type":"function"},{"doc":"","ref":"Posexional.File.html#new/2","title":"Posexional.File.new/2","type":"function"},{"doc":"","ref":"Posexional.File.html#read/2","title":"Posexional.File.read/2","type":"function"},{"doc":"","ref":"Posexional.File.html#stream/2","title":"Posexional.File.stream/2","type":"function"},{"doc":"creates a file from values","ref":"Posexional.File.html#write/2","title":"Posexional.File.write/2","type":"function"},{"doc":"iex> write(\n    ...>   Posexional.File.new([ Posexional.Row.new(:row_test, [ Posexional.Field.Value.new(:test1, 5) ]) ]),\n    ...>   [row_test: [test1: \"test\"], row_test: [test1: \"t\"]]\n    ...> )\n    \"test \\nt    \"\n\n    iex> write(\n    ...>   Posexional.File.new([ Posexional.Row.new(:row_test, [ Posexional.Field.Value.new(:test1, 5) ]) ]),\n    ...>   [row_test: [test1: \"test\"], ne: [test1: \"t\"]]\n    ...> )\n    ** (RuntimeError) row ne not found","ref":"Posexional.File.html#write/2-examples","title":"Examples - Posexional.File.write/2","type":"function"},{"doc":"","ref":"Posexional.File.html#write_path!/3","title":"Posexional.File.write_path!/3","type":"function"},{"doc":"","ref":"Posexional.File.html#t:t/0","title":"Posexional.File.t/0","type":"type"},{"doc":"","ref":"Posexional.Protocol.FieldLength.html","title":"Posexional.Protocol.FieldLength","type":"protocol"},{"doc":"returns the field length","ref":"Posexional.Protocol.FieldLength.html#length/1","title":"Posexional.Protocol.FieldLength.length/1","type":"function"},{"doc":"","ref":"Posexional.Protocol.FieldLength.html#t:t/0","title":"Posexional.Protocol.FieldLength.t/0","type":"type"},{"doc":"","ref":"Posexional.Protocol.FieldName.html","title":"Posexional.Protocol.FieldName","type":"protocol"},{"doc":"returns the field name","ref":"Posexional.Protocol.FieldName.html#name/1","title":"Posexional.Protocol.FieldName.name/1","type":"function"},{"doc":"","ref":"Posexional.Protocol.FieldName.html#t:t/0","title":"Posexional.Protocol.FieldName.t/0","type":"type"},{"doc":"","ref":"Posexional.Protocol.FieldRead.html","title":"Posexional.Protocol.FieldRead","type":"protocol"},{"doc":"returns the value of a field, given it's positional representation","ref":"Posexional.Protocol.FieldRead.html#read/2","title":"Posexional.Protocol.FieldRead.read/2","type":"function"},{"doc":"","ref":"Posexional.Protocol.FieldRead.html#t:t/0","title":"Posexional.Protocol.FieldRead.t/0","type":"type"},{"doc":"","ref":"Posexional.Protocol.FieldSize.html","title":"Posexional.Protocol.FieldSize","type":"protocol"},{"doc":"returns the field name","ref":"Posexional.Protocol.FieldSize.html#size/1","title":"Posexional.Protocol.FieldSize.size/1","type":"function"},{"doc":"","ref":"Posexional.Protocol.FieldSize.html#t:t/0","title":"Posexional.Protocol.FieldSize.t/0","type":"type"},{"doc":"","ref":"Posexional.Protocol.FieldWrite.html","title":"Posexional.Protocol.FieldWrite","type":"protocol"},{"doc":"returns the field to be inserted in the positional file","ref":"Posexional.Protocol.FieldWrite.html#write/2","title":"Posexional.Protocol.FieldWrite.write/2","type":"function"},{"doc":"","ref":"Posexional.Protocol.FieldWrite.html#t:t/0","title":"Posexional.Protocol.FieldWrite.t/0","type":"type"},{"doc":"this module represent a row in a positional file","ref":"Posexional.Row.html","title":"Posexional.Row","type":"module"},{"doc":"","ref":"Posexional.Row.html#add_field/2","title":"Posexional.Row.add_field/2","type":"function"},{"doc":"","ref":"Posexional.Row.html#add_fields/2","title":"Posexional.Row.add_fields/2","type":"function"},{"doc":"merge fields from another row","ref":"Posexional.Row.html#fields_from/2","title":"Posexional.Row.fields_from/2","type":"function"},{"doc":"finds a field in the row by its name","ref":"Posexional.Row.html#find_field/2","title":"Posexional.Row.find_field/2","type":"function"},{"doc":"iex> Posexional.Row.new(:row_test, []) |> Posexional.Row.find_field(:test)\n    nil\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test, 5)]) |> Posexional.Row.find_field(:test)\n    Posexional.Field.Value.new(:test, 5)\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test, 5), Posexional.Field.Value.new(:test2, 5)])\n    ...>   |> Posexional.Row.find_field(:test2)\n    Posexional.Field.Value.new(:test2, 5)","ref":"Posexional.Row.html#find_field/2-examples","title":"Examples - Posexional.Row.find_field/2","type":"function"},{"doc":"calculate the row total length based on the passed fields","ref":"Posexional.Row.html#length/1","title":"Posexional.Row.length/1","type":"function"},{"doc":"iex> Posexional.Row.new(:row_test, [])\n    ...>   |> Posexional.Row.length\n    0\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)])\n    ...>   |> Posexional.Row.length\n    30","ref":"Posexional.Row.html#length/1-examples","title":"Examples - Posexional.Row.length/1","type":"function"},{"doc":"","ref":"Posexional.Row.html#manage_counters/2","title":"Posexional.Row.manage_counters/2","type":"function"},{"doc":"","ref":"Posexional.Row.html#new/3","title":"Posexional.Row.new/3","type":"function"},{"doc":"Given a row and a field name calculate the field offset","ref":"Posexional.Row.html#offset/2","title":"Posexional.Row.offset/2","type":"function"},{"doc":"iex> Posexional.Row.new(:test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)])\n    ...>     |> Posexional.Row.offset(:test1)\n    1\n\n    iex> Posexional.Row.new(:test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)])\n    ...>    |> Posexional.Row.offset(:test2)\n    11\n\n    iex> Posexional.Row.new(:test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)])\n    ...>     |> Posexional.Row.offset(:test_not_existent)\n    ** (ArgumentError) the field test_not_existent doesn't exists\n\n    iex> Posexional.Row.new(:test, [])\n    ...>     |> Posexional.Row.offset(:test)\n    nil","ref":"Posexional.Row.html#offset/2-examples","title":"Examples - Posexional.Row.offset/2","type":"function"},{"doc":"read a positional file row and convert it back to a keyword list of values","ref":"Posexional.Row.html#read/2","title":"Posexional.Row.read/2","type":"function"},{"doc":"outputs a row","ref":"Posexional.Row.html#write/2","title":"Posexional.Row.write/2","type":"function"},{"doc":"iex> Posexional.Row.new(:row_test, []) |> Posexional.Row.write([test: \"test\"])\n    {:ok, \"\"}\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 5), Posexional.Field.Value.new(:test2, 10)])\n    ...>   |> Posexional.Row.write([test1: \"test1\", test2: \"test2\"])\n    {:ok, \"test1test2     \"}\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 5), Posexional.Field.Value.new(:test2, 10)])\n    ...>   |> Posexional.Row.write([test1: \"test1\", non_existent: \"test2\"])\n    {:ok, \"test1          \"}\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 6)])\n    ...>   |> Posexional.Row.write([test1: \"test1\", not_configured: \"test2\"])\n    {:ok, \"test1 \"}\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 5)])\n    ...>   |> Posexional.Row.write([not_configured: \"test2\", another: \"test3\"])\n    {:ok, \"     \"}\n\n    iex> Posexional.Row.new(:row_test, [Posexional.Field.Empty.new(5)])\n    ...>   |> Posexional.Row.write([])\n    {:ok, \"     \"}","ref":"Posexional.Row.html#write/2-examples","title":"Examples - Posexional.Row.write/2","type":"function"},{"doc":"","ref":"Posexional.Row.html#t:t/0","title":"Posexional.Row.t/0","type":"type"},{"doc":"macros for defining file modules","ref":"PosexionalFile.html","title":"PosexionalFile","type":"module"},{"doc":"add use Posexional on top of an elixir module to use macros to define fields","ref":"PosexionalFile.html#__using__/1","title":"PosexionalFile.__using__/1","type":"macro"},{"doc":"","ref":"PosexionalFile.html#row/1","title":"PosexionalFile.row/1","type":"macro"},{"doc":"define a row of a positional file","ref":"PosexionalFile.html#row/2","title":"PosexionalFile.row/2","type":"macro"},{"doc":"","ref":"PosexionalFile.html#row/3","title":"PosexionalFile.row/3","type":"macro"},{"doc":"macros for defining row modules","ref":"PosexionalRow.html","title":"PosexionalRow","type":"module"},{"doc":"add use Posexional on top of an elixir module to use macros to define fields","ref":"PosexionalRow.html#__using__/1","title":"PosexionalRow.__using__/1","type":"macro"},{"doc":"add an empty field","ref":"PosexionalRow.html#empty/2","title":"PosexionalRow.empty/2","type":"macro"},{"doc":"add a field","ref":"PosexionalRow.html#field/4","title":"PosexionalRow.field/4","type":"macro"},{"doc":"add a field with a fixed value","ref":"PosexionalRow.html#fixed_value/1","title":"PosexionalRow.fixed_value/1","type":"macro"},{"doc":"sets the row matcher","ref":"PosexionalRow.html#guesser/1","title":"PosexionalRow.guesser/1","type":"macro"},{"doc":"add all fields from another row module","ref":"PosexionalRow.html#import_fields_from/1","title":"PosexionalRow.import_fields_from/1","type":"macro"},{"doc":"sets the row name, if no name is provided the module name will be used","ref":"PosexionalRow.html#name/1","title":"PosexionalRow.name/1","type":"macro"},{"doc":"add a field with a progressive_number value","ref":"PosexionalRow.html#progressive_number/3","title":"PosexionalRow.progressive_number/3","type":"macro"},{"doc":"sets the row separator","ref":"PosexionalRow.html#separator/1","title":"PosexionalRow.separator/1","type":"macro"},{"doc":"add a value field","ref":"PosexionalRow.html#value/3","title":"PosexionalRow.value/3","type":"macro"},{"doc":"# Posexional\n\n[![Build Status](https://drone-1.prima.it/api/badges/primait/posexional/status.svg)](https://drone-1.prima.it/primait/posexional)\n[![Module Version](https://img.shields.io/hexpm/v/posexional.svg)](https://hex.pm/packages/posexional)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/posexional/)\n[![Total Download](https://img.shields.io/hexpm/dt/posexional.svg)](https://hex.pm/packages/posexional)\n[![License](https://img.shields.io/hexpm/l/posexional.svg)](https://hex.pm/packages/posexional)\n[![Last Updated](https://img.shields.io/github/last-commit/primait/posexional.svg)](https://github.com/primait/posexional/commits/master)\n\nPosexional is a library to manage positional files in Elixir.","ref":"readme.html","title":"Posexional","type":"extras"},{"doc":"Add `posexional` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:posexional, \"~> 0.4\"}]\nend\n```","ref":"readme.html#installation","title":"Installation - Posexional","type":"extras"},{"doc":"<!--MDOC !-->\n\nPositional files are the most bad and **terrifying file format** you ever want\nto work with, believe me, it's utter crap. Still some *web services* use this\nformat to expose data services, and if you are here reading this, probably you\nare in luck like us!\n\nA positional file has a *format specification* like this:\n\n| field name | offset | length | notes                                           |\n|------------|--------|--------|-------------------------------------------------|\n| code       | 0      | 5      | request code fill with 0 and align on the right |\n| prog       | 0      | 5      | progressive number (fill with spaces)           |\n| type       | 21     | 2      | AA                                              |\n| number     | 24     | 2      | 01                                              |\n| name       | 27     | 10     | person name (fill with -)                       |\n| future use | 38     | 2      | leave empty                                     |\n| end        | 41     | 1      | !                                               |\n\nThe expected results should be something like this:\n\n    000B1    1AA01george----  !\n    000B2    2AA01john------  !\n    000B3    3AA01ringo-----  !\n    000B4    4AA01paul------  !\n\n\nCool uh?\n\nWith Posexional you can produce this file by defining a module that use\nPosexional.\n\n```elixir\ndefmodule BeatlesFile do\n  use PosexionalFile\n\n  @separator \"\\\\n\"\n\n  row :beatles do\n    value :code, 5, filler: ?0, alignment: :right\n    progressive_number :code, 5\n    fixed_value \"AA\"\n    fixed_value \"01\"\n    value :name, 10, filler: ?-\n    empty 2\n    fixed_value \"!\"\n  end\nend\n```\n\nAnd then use it in your code.\n\n```elixir\nBeatlesFile.write([\n  beatles: [code: \"B1\", name: \"george\"],\n  beatles: [code: \"B2\", name: \"john\"],\n  beatles: [code: \"B2\", name: \"ringo\"],\n  beatles: [code: \"B2\", name: \"paul\"]\n])\n```\n\nIn the first part we **define the structure** inside a module. We are not\nsaying what the content or the number of rows there will be, we are just saying\nthat there is a row called `:beatles` with the structure declared by the fields.\n\nThen we can call `BeatlesFile.write/1`, we pass the data that should be written\nin the fields. **Just the relevant data**, The empty fields, the fixed values\nor the progressive number is managed by the library itself.\n\nThe `write/1` function accept a keyword list with the row name as key, and a\nkeyword list of {field name, field value} of data. If some data is bigger than\nthe field size an error is thrown\n\nWith the same exact module, we can even **read a positional file** by calling\n`read/1` and passing a binary string of the file content.\n\nThere is only one thing to notice, when we write a file we **can be\ndeclarative** and say what row we want to write, as well as the data we want in\nit. On the other hand, while reading a positional file, we don't know which\nrows we are reading, so we need to tell in some way to every row how it is\nrecognized, so that the parser is able to do its job.\n\nSince we only have a row type (the beatles one) we can just say to the module\nto always match the beatles row, as simple as:\n\n```elixir\ndefmodule BeatlesFile do\n  use PosexionalFile\n\n  @separator \"\\\\n\"\n\n  row :beatles, :always do # add :always here to always match this row while reading\n    value :code, 5, filler: ?0, alignment: :right\n    progressive_number :code, 5\n    fixed_value \"AA\"\n    fixed_value \"01\"\n    value :name, 10, filler: ?-\n    empty 2\n    fixed_value \"!\"\n  end\nend\n```\n\n`:always` is a special type that always match, you could also pass `:never`\n(not so useful!) and, to gain total control over the choice, a function with a\nsingle argument(the full row content) to match some data inside of it.\n\nNow we are able to parse a positional file.\n\n```elixir\n\"000B1    1AA01george----  !\\n000B2    2AA01john------  !\\n000B2    3AA01ringo-----  !\\n000B2    4AA01paul------  !\"\n|> BeatlesFile.read\n|> IO.puts\n```\n\nThis is the output.\n\n```elixir\n[beatles: [code: \"B1\", code: 1, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"george\", fixed_value: \"!\"],\n beatles: [code: \"B2\", code: 2, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"john\", fixed_value: \"!\"],\n beatles: [code: \"B2\", code: 3, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"ringo\", fixed_value: \"!\"],\n beatles: [code: \"B2\", code: 4, fixed_value: \"AA\", fixed_value: \"01\",\n  name: \"paul\", fixed_value: \"!\"]]\n```\n\nPositional files cool again!!! Well no...they still sucks...but a little less.\n\n<!--MDOC !-->","ref":"readme.html#usage","title":"Usage - Posexional","type":"extras"},{"doc":"Thank your for considering helping with this project. Please see\n`CONTRIBUTING.md` file for contributing to this project.","ref":"readme.html#contributing","title":"Contributing - Posexional","type":"extras"},{"doc":"MIT License. Copyright (c) 2015-2020 Prima.it","ref":"readme.html#license","title":"License - Posexional","type":"extras"},{"doc":"# Contributing\n\nWhen contributing to this repository, please first discuss the change you wish to make via issue,\nemail, or any other method with the owners of this repository before making a change.","ref":"contributing.html","title":"Contributing","type":"extras"},{"doc":"1. Ensure any install or build dependencies are removed before the end of the layer when doing a\n   build.\n2. Update the README.md with details of any structural change, this includes new environment\n   variables, exposed ports, useful file locations and container parameters.\n3. Increase the version numbers in any examples files and the README.md to the new version that this\n   Pull Request would represent. The versioning scheme we use is [SemVer](http://semver.org/).\n4. Before submitting a Pull Request run `mix test` and `mix check` and ensure that they complete without any errors.\n5. You should request a review by at least one repo maintainer.","ref":"contributing.html#pull-request-process","title":"Pull Request Process - Contributing","type":"extras"}]}